library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity ext_interface is
    Port ( clk : in STD_LOGIC;
           sck : in STD_LOGIC;
           sin : in STD_LOGIC;
           ldcp : in STD_LOGIC;           
           inc : out STD_LOGIC;
           dec : out STD_LOGIC;
           key : out STD_LOGIC_VECTOR (1 downto 0)        
    ); 
end ext_interface;

architecture Behavioral of ext_interface is
 type states is (idle, pulse, load);
 signal st: states;
 signal sreg: STD_LOGIC_VECTOR(7 downto 0);
 signal reg : STD_LOGIC_VECTOR(7 downto 0);
 
 -- rotary encoder states
 type states_rot is (idle1, s1, s2, wt);
 signal str: states_rot;
 signal sck1, sin1, ld1: std_logic;
 
begin
 p1: process(clk)
 begin
  if rising_edge(clk) then
    sck1 <= sck;
    sin1 <= sin;
    ld1 <= ldcp;
    
    case st is
      when idle =>
        if sck1='1' then
          st <= pulse;
          sreg <= sreg(6 downto 0) & sin1;
        elsif ldcp='0' then
          st <= load;
        end if;
        
      when pulse =>
        if sck1='0' then
          st <= idle;            
        end if;
                
      when others =>  -- load        
        if ld1='1' then
          reg <= sreg;
          st <= idle;
        end if;
    end case;
  end if;
 end process;

 p2: process(clk)
 begin
  if rising_edge(clk) then
    inc <= '0';
    dec <= '0';
    case str is
      when idle1 =>
        if reg(2 downto 1)="01" then
          str <= s1;
        elsif reg(2 downto 1)="10" then
          str <= s2;
        end if;
        
      when s1 =>
        case reg(2 downto 1) is
            when "00" => str <= wt;
            when "10" => str <= idle1;
            when "11" => dec <= '1';        
              str <= wt; 
            when others => null;
        end case;
        
      when s2 =>
        case reg(2 downto 1) is
          when "00" => str <= wt;
          when "01" => str <= idle1;
          when "11" => inc <= '1';         
            str <= wt; 
          when others => null;
      end case;

      when others => -- wt
        if reg(2 downto 1)="00" then
          str <= idle1;
        end if;
    end case;  
  end if;
 end process;

 key <= reg(4 downto 3);

end Behavioral;
